package java.nio.file;

import sun.nio.fs.DefaultFileSystemProvider;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Collections;
import java.util.ServiceLoader;
import java.io.IOException;
import java.util.Map;
import java.util.Iterator;
import java.nio.file.spi.FileSystemProvider;
import java.net.URI;

public final class FileSystems
{
    private FileSystems() {
    }
    
    public static FileSystem getDefault() {
        return FileSystems.DefaultFileSystemHolder.defaultFileSystem;
    }
    
    public static FileSystem getFileSystem(URI uri) {
        String scheme = uri.getScheme();
        for (FileSystemProvider fileSystemProvider : FileSystemProvider.installedProviders()) {
            if (scheme.equalsIgnoreCase(fileSystemProvider.getScheme())) {
                return fileSystemProvider.getFileSystem(uri);
            }
            continue;
        }
        throw new ProviderNotFoundException("Provider \"" + scheme + "\" not found");
    }
    
    public static FileSystem newFileSystem(URI uri, Map<String, ?> map) throws IOException {
        return newFileSystem(uri, map, null);
    }
    
    public static FileSystem newFileSystem(URI uri, Map<String, ?> map, ClassLoader loader) throws IOException {
        String scheme = uri.getScheme();
        for (FileSystemProvider fileSystemProvider : FileSystemProvider.installedProviders()) {
            if (scheme.equalsIgnoreCase(fileSystemProvider.getScheme())) {
                return fileSystemProvider.newFileSystem(uri, map);
            }
            continue;
        }
        if (loader != null) {
            for (FileSystemProvider fileSystemProvider2 : ServiceLoader.load(FileSystemProvider.class, loader)) {
                if (scheme.equalsIgnoreCase(fileSystemProvider2.getScheme())) {
                    return fileSystemProvider2.newFileSystem(uri, map);
                }
                continue;
            }
        }
        throw new ProviderNotFoundException("Provider \"" + scheme + "\" not found");
    }
    
    public static FileSystem newFileSystem(Path path, ClassLoader loader) throws IOException {
        if (path == null) {
            throw new NullPointerException();
        }
        Map<String, Object> emptyMap = Collections.emptyMap();
        for (FileSystemProvider fileSystemProvider : FileSystemProvider.installedProviders()) {
            try {
                return fileSystemProvider.newFileSystem(path, emptyMap);
            }
            catch (UnsupportedOperationException ex) {}
        }
        if (loader != null) {
            for (FileSystemProvider fileSystemProvider2 : ServiceLoader.load(FileSystemProvider.class, loader)) {
                try {
                    return fileSystemProvider2.newFileSystem(path, emptyMap);
                }
                catch (UnsupportedOperationException ex2) {}
            }
        }
        throw new ProviderNotFoundException("Provider not found");
    }
    
    private static class DefaultFileSystemHolder
    {
        static final FileSystem defaultFileSystem;
        
        private static FileSystem defaultFileSystem() {
            return ((FileSystemProvider)AccessController.doPrivileged(new PrivilegedAction<FileSystemProvider>() {
                @Override
                public FileSystemProvider run() {
                    return FileSystems.DefaultFileSystemHolder.access$000();
                }
            })).getFileSystem(URI.create("file:///"));
        }
        
        private static FileSystemProvider getDefaultProvider() {
            FileSystemProvider create = DefaultFileSystemProvider.create();
            String property = System.getProperty("java.nio.file.spi.DefaultFileSystemProvider");
            if (property != null) {
                for (String name : property.split(",")) {
                    try {
                        create = (FileSystemProvider)Class.forName(name, true, ClassLoader.getSystemClassLoader()).getDeclaredConstructor(new Class[] { FileSystemProvider.class }).newInstance(new Object[] { create });
                        if (!create.getScheme().equals("file")) {
                            throw new Error("Default provider must use scheme 'file'");
                        }
                    }
                    catch (Exception cause) {
                        throw new Error(cause);
                    }
                }
                return create;
            }
            return create;
        }
        
        static FileSystemProvider access$000() {
            return getDefaultProvider();
        }
        
        static {
            defaultFileSystem = defaultFileSystem();
        }
    }
}
